@@ -18,7 +18,7 @@ import sys
 import threading
 from datetime import datetime
 from functools import lru_cache
-from typing import Generator, Iterable, List, Optional
+from typing import Generator, Iterable, List, NewType, Optional, Tuple
 
 import hglib
 import lmdb
@@ -30,6 +30,8 @@ from bugbug.utils import LMDBDict
 
 logger = logging.getLogger(__name__)
 
+CommitDict = NewType("CommitDict", dict)
+
 code_analysis_server: Optional[rust_code_analysis_server.RustCodeAnalysisServer] = None
 
 hg_servers = list()
@@ -214,14 +216,14 @@ class Commit:
             setattr(self, f"{exp_str}max", exp_max)
             setattr(self, f"{exp_str}min", exp_min)
 
-    def to_dict(self):
+    def to_dict(self) -> CommitDict:
         d = self.__dict__
         for f in ["file_copies"]:
             del d[f]
         d["types"] = list(d["types"])
         d["pushdate"] = str(d["pushdate"])
         d["date"] = str(d["date"])
-        return d
+        return CommitDict(d)
 
 
 def get_directories(files):
@@ -245,11 +247,11 @@ def is_wptsync(commit: dict) -> bool:
 
 
 def filter_commits(
-    commits: Iterable[dict],
+    commits: Iterable[CommitDict],
     include_no_bug: bool = False,
     include_backouts: bool = False,
     include_ignored: bool = False,
-) -> Generator[dict, None, None]:
+) -> Generator[CommitDict, None, None]:
     for commit in commits:
         if not include_ignored and commit["ignored"]:
             continue
@@ -267,7 +269,7 @@ def get_commits(
     include_no_bug: bool = False,
     include_backouts: bool = False,
     include_ignored: bool = False,
-) -> Generator[dict, None, None]:
+) -> Generator[CommitDict, None, None]:
     return filter_commits(
         db.read(COMMITS_DB),
         include_no_bug=include_no_bug,
@@ -623,20 +625,16 @@ def _transform(commit):
     return transform(HG, REPO_DIR, commit)
 
 
-def hg_log(hg, revs):
+def hg_log(hg: hglib.client, revs: List[bytes]) -> List[Commit]:
     template = "{node}\\0{author}\\0{desc}\\0{date|hgdate}\\0{bug}\\0{backedoutby}\\0{author|email}\\0{pushdate|hgdate}\\0{reviewers}\\0{backsoutnodes}\\0"
 
     args = hglib.util.cmdbuilder(
-        b"log",
-        template=template,
-        no_merges=True,
-        rev=revs[0] + b":" + revs[-1],
-        branch="tip",
+        b"log", template=template, no_merges=True, rev=revs, branch="tip",
     )
     x = hg.rawcommand(args)
     out = x.split(b"\x00")[:-1]
 
-    revs = []
+    commits = []
     for rev in hglib.util.grouper(template.count("\\0"), out):
         assert b" " in rev[3]
         date = datetime.utcfromtimestamp(float(rev[3].split(b" ", 1)[0]))
@@ -662,7 +660,7 @@ def hg_log(hg, revs):
             else []
         )
 
-        revs.append(
+        commits.append(
             Commit(
                 node=sys.intern(rev[0].decode("ascii")),
                 author=sys.intern(rev[1].decode("utf-8")),
@@ -677,10 +675,10 @@ def hg_log(hg, revs):
             )
         )
 
-    return revs
+    return commits
 
 
-def _hg_log(revs):
+def _hg_log(revs: List[bytes]) -> List[Commit]:
     return hg_log(thread_local.hg, revs)
 
 
@@ -766,21 +764,24 @@ def calculate_experiences(commits, first_pushdate, save=True):
 
     def get_experience(exp_type, commit_type, item, day, default):
         key = get_key(exp_type, commit_type, item)
-        if key not in experiences:
-            experiences[key] = utils.ExpQueue(day, EXPERIENCE_TIMESPAN + 1, default)
-        return experiences[key]
+        try:
+            return experiences[key]
+        except KeyError:
+            queue = utils.ExpQueue(day, EXPERIENCE_TIMESPAN + 1, default)
+            experiences[key] = queue
+            return queue
 
     def update_experiences(experience_type, day, items):
-        for commit_type in ["", "backout"]:
-            exp_queues = [
+        for commit_type in ("", "backout"):
+            exp_queues = tuple(
                 get_experience(experience_type, commit_type, item, day, 0)
                 for item in items
-            ]
-            total_exps = [exp_queues[i][day] for i in range(len(items))]
-            timespan_exps = [
+            )
+            total_exps = tuple(exp_queues[i][day] for i in range(len(items)))
+            timespan_exps = tuple(
                 exp - exp_queues[i][day - EXPERIENCE_TIMESPAN]
                 for exp, i in zip(total_exps, range(len(items)))
-            ]
+            )
 
             total_exps_sum = sum(total_exps)
             timespan_exps_sum = sum(timespan_exps)
@@ -813,21 +814,21 @@ def calculate_experiences(commits, first_pushdate, save=True):
                     exp_queues[i][day] = total_exps[i] + 1
 
     def update_complex_experiences(experience_type, day, items):
-        for commit_type in ["", "backout"]:
-            exp_queues = [
+        for commit_type in ("", "backout"):
+            exp_queues = tuple(
                 get_experience(experience_type, commit_type, item, day, tuple())
                 for item in items
-            ]
-            all_commit_lists = [exp_queues[i][day] for i in range(len(items))]
-            before_commit_lists = [
+            )
+            all_commit_lists = tuple(exp_queues[i][day] for i in range(len(items)))
+            before_commit_lists = tuple(
                 exp_queues[i][day - EXPERIENCE_TIMESPAN] for i in range(len(items))
-            ]
-            timespan_commit_lists = [
+            )
+            timespan_commit_lists = tuple(
                 commit_list[len(before_commit_list) :]
                 for commit_list, before_commit_list in zip(
                     all_commit_lists, before_commit_lists
                 )
-            ]
+            )
 
             all_commits = set(sum(all_commit_lists, tuple()))
             timespan_commits = set(sum(timespan_commit_lists, tuple()))
@@ -892,7 +893,7 @@ def calculate_experiences(commits, first_pushdate, save=True):
 
         # When a file is moved/copied, copy original experience values to the copied path.
         for orig, copied in commit.file_copies.items():
-            for commit_type in ["", "backout"]:
+            for commit_type in ("", "backout"):
                 orig_key = get_key("file", commit_type, orig)
                 if orig_key in experiences:
                     experiences[get_key("file", commit_type, copied)] = copy.deepcopy(
@@ -908,7 +909,7 @@ def calculate_experiences(commits, first_pushdate, save=True):
             and len(commit.backsout) == 0
             and commit.bug_id is not None
         ):
-            update_experiences("author", day, [commit.author])
+            update_experiences("author", day, (commit.author,))
             update_experiences("reviewer", day, commit.reviewers)
 
             update_complex_experiences("file", day, commit.files)
@@ -923,7 +924,7 @@ def set_commits_to_ignore(hg: hglib.client, repo_dir: str, commits: List[Commit]
     ignore_revs_content = hg.cat(
         [os.path.join(repo_dir, ".hg-annotate-ignore-revs").encode("ascii")], rev=b"-1"
     ).decode("utf-8")
-    ignore_revs = set(l[:40] for l in ignore_revs_content.splitlines())
+    ignore_revs = set(line[:40] for line in ignore_revs_content.splitlines())
 
     for commit in commits:
         commit.ignored = (
@@ -967,10 +968,7 @@ def hg_log_multi(repo_dir, revs):
     threads_num = os.cpu_count() + 1
     REVS_COUNT = len(revs)
     CHUNK_SIZE = int(math.ceil(REVS_COUNT / threads_num))
-    revs_groups = [
-        (revs[i], revs[min(i + CHUNK_SIZE, REVS_COUNT) - 1])
-        for i in range(0, REVS_COUNT, CHUNK_SIZE)
-    ]
+    revs_groups = [revs[i : i + CHUNK_SIZE] for i in range(0, REVS_COUNT, CHUNK_SIZE)]
 
     with concurrent.futures.ThreadPoolExecutor(
         initializer=_init_thread, initargs=(repo_dir,), max_workers=threads_num
@@ -995,13 +993,13 @@ def get_first_pushdate(repo_dir):
 def download_commits(
     repo_dir: str,
     rev_start: str = None,
-    revs: List[str] = None,
+    revs: List[bytes] = None,
     save: bool = True,
     use_single_process: bool = False,
     include_no_bug: bool = False,
     include_backouts: bool = False,
     include_ignored: bool = False,
-) -> List[dict]:
+) -> Tuple[CommitDict, ...]:
     assert revs is not None or rev_start is not None
 
     with hglib.open(repo_dir) as hg:
@@ -1009,7 +1007,7 @@ def download_commits(
             revs = get_revs(hg, rev_start)
             if len(revs) == 0:
                 logger.info("No commits to analyze")
-                return []
+                return tuple()
 
         first_pushdate = get_first_pushdate(repo_dir)
 
@@ -1040,11 +1038,11 @@ def download_commits(
             ) as executor:
                 commits = executor.map(_transform, commits, chunksize=64)
                 commits = tqdm(commits, total=commits_num)
-                commits = list(commits)
+                commits = tuple(commits)
         else:
             get_component_mapping()
 
-            commits = [transform(hg, repo_dir, c) for c in commits]
+            commits = tuple(transform(hg, repo_dir, c) for c in commits)
 
             close_component_mapping()
 
@@ -1054,12 +1052,12 @@ def download_commits(
 
     logger.info("Applying final commits filtering...")
 
-    commits = [commit.to_dict() for commit in commits]
+    commits = tuple(commit.to_dict() for commit in commits)
 
     if save:
         db.append(COMMITS_DB, commits)
 
-    return list(
+    return tuple(
         filter_commits(
             commits,
             include_no_bug=include_no_bug,
@@ -1084,7 +1082,19 @@ def clean(hg, repo_dir):
             raise
 
 
-def clone(repo_dir, url="https://hg.mozilla.org/mozilla-central", update=False):
+def _run_hg_cmd(repo_dir, cmd, *args, **kwargs):
+    cmd = hglib.util.cmdbuilder(cmd, *args, **kwargs,)
+
+    cmd.insert(0, hglib.HGPATH)
+
+    subprocess.run(cmd, cwd=repo_dir, check=True)
+
+
+def clone(
+    repo_dir: str,
+    url: str = "https://hg.mozilla.org/mozilla-central",
+    update: bool = False,
+) -> None:
     try:
         with hglib.open(repo_dir) as hg:
             clean(hg, repo_dir)
@@ -1103,10 +1113,11 @@ def clone(repo_dir, url="https://hg.mozilla.org/mozilla-central", update=False):
 
         return
     except hglib.error.ServerError as e:
-        if "abort: repository" not in str(e) and b"not found" not in str(e):
+        if "abort: repository" not in str(e) and "not found" not in str(e):
             raise
 
-    cmd = hglib.util.cmdbuilder(
+    _run_hg_cmd(
+        None,
         "robustcheckout",
         url,
         repo_dir,
@@ -1117,23 +1128,26 @@ def clone(repo_dir, url="https://hg.mozilla.org/mozilla-central", update=False):
         noupdate=not update,
     )
 
-    cmd.insert(0, hglib.HGPATH)
-
-    subprocess.run(cmd, check=True)
-
     logger.info(f"{repo_dir} cloned")
 
 
-def apply_stack(repo_dir, stack, branch):
-    """Apply a stack of patches on a repository"""
-    assert len(stack) > 0, "Empty stack"
+def pull(repo_dir: str, branch: str, revision: str) -> None:
+    """Pull a revision from a branch of a remote repository into a local repository"""
 
-    with hglib.open(repo_dir) as hg:
-        hg.pull(
-            source=f"https://hg.mozilla.org/{branch}/".encode("ascii"),
-            rev=stack[-1]["pushhead"].encode("ascii"),
+    def do_pull() -> None:
+        _run_hg_cmd(
+            repo_dir,
+            "pull",
+            f"https://hg.mozilla.org/{branch}/".encode("ascii"),
+            r=revision.encode("ascii"),
+            debug=True,
         )
-        return [rev["node"].encode("ascii") for rev in stack]
+
+    try:
+        do_pull()
+    except subprocess.CalledProcessError:
+        _run_hg_cmd(repo_dir, "recover")
+        do_pull()
 
 
 if __name__ == "__main__":
