@@ -21,6 +21,7 @@ from sklearn.neighbors import NearestNeighbors
 from tqdm import tqdm
 
 from bugbug import bugzilla, feature_cleanup
+from bugbug.models.duplicate import DuplicateModel
 from bugbug.utils import download_check_etag, zstd_decompress
 
 OPT_MSG_MISSING = (
@@ -80,7 +81,13 @@ def download_and_load_similarity_model(model_name):
 
 
 class BaseSimilarity(abc.ABC):
-    def __init__(self, cleanup_urls=True, nltk_tokenizer=False):
+    def __init__(
+        self,
+        cleanup_urls=True,
+        nltk_tokenizer=False,
+        confidence_threshold=0.8,
+        end_to_end=False,
+    ):
         self.cleanup_functions = [
             feature_cleanup.responses(),
             feature_cleanup.hex(),
@@ -93,6 +100,11 @@ class BaseSimilarity(abc.ABC):
             self.cleanup_functions.append(feature_cleanup.url())
 
         self.nltk_tokenizer = nltk_tokenizer
+        self.confidence_threshold = confidence_threshold
+
+        self.duplicatemodel = (
+            DuplicateModel.load("duplicatemodel") if end_to_end else None
+        )
 
     def get_text(self, bug, all_comments=False):
         if all_comments:
@@ -102,6 +114,29 @@ class BaseSimilarity(abc.ABC):
 
         return "{} {}".format(bug["summary"], comments)
 
+    def get_similar_bugs(self, bug):
+        similar_bug_ids = self.search_similar_bugs(bug)
+        if self.duplicatemodel:
+            similar_bugs = [
+                bug for bug in bugzilla.get_bugs() if bug["id"] in similar_bug_ids
+            ]
+            bug_couples = [(bug, similar_bug) for similar_bug in similar_bugs]
+            probs_bug_couples = sorted(
+                zip(
+                    self.duplicatemodel.classify(bug_couples, probabilities=True),
+                    bug_couples,
+                ),
+                key=lambda v: -v[0][1],
+            )
+
+            similar_bug_ids = [
+                similar_bug["id"]
+                for prob, (bug, similar_bug) in probs_bug_couples
+                if prob[1] > self.confidence_threshold
+            ]
+
+        return similar_bug_ids
+
     def text_preprocess(self, text, stemming=True, lemmatization=False, join=False):
 
         for func in self.cleanup_functions:
@@ -226,8 +261,14 @@ class BaseSimilarity(abc.ABC):
 
 
 class LSISimilarity(BaseSimilarity):
-    def __init__(self, cleanup_urls=True, nltk_tokenizer=False):
-        super().__init__(cleanup_urls=cleanup_urls, nltk_tokenizer=nltk_tokenizer)
+    def __init__(
+        self, cleanup_urls=True, nltk_tokenizer=False, confidence_threshold=0.8
+    ):
+        super().__init__(
+            cleanup_urls=cleanup_urls,
+            nltk_tokenizer=nltk_tokenizer,
+            confidence_threshold=confidence_threshold,
+        )
         self.corpus = []
 
         for bug in bugzilla.get_bugs():
@@ -256,7 +297,7 @@ class LSISimilarity(BaseSimilarity):
             output_prefix="simdata.shdat", corpus=corpus_lsi, num_features=300
         )
 
-    def get_similar_bugs(self, query, k=10):
+    def search_similar_bugs(self, query, k=10):
         query_summary = "{} {}".format(query["summary"], query["comments"][0]["text"])
         query_summary = self.text_preprocess(query_summary)
 
@@ -286,8 +327,13 @@ class NeighborsSimilarity(BaseSimilarity):
         vectorizer=TfidfVectorizer(),
         cleanup_urls=True,
         nltk_tokenizer=False,
+        confidence_threshold=0.8,
     ):
-        super().__init__(cleanup_urls=cleanup_urls, nltk_tokenizer=nltk_tokenizer)
+        super().__init__(
+            cleanup_urls=cleanup_urls,
+            nltk_tokenizer=nltk_tokenizer,
+            confidence_threshold=confidence_threshold,
+        )
         self.vectorizer = vectorizer
         self.similarity_calculator = NearestNeighbors(n_neighbors=k)
         text = []
@@ -300,7 +346,7 @@ class NeighborsSimilarity(BaseSimilarity):
         self.vectorizer.fit(text)
         self.similarity_calculator.fit(self.vectorizer.transform(text))
 
-    def get_similar_bugs(self, query):
+    def search_similar_bugs(self, query):
 
         processed_query = self.vectorizer.transform([self.get_text(query)])
         _, indices = self.similarity_calculator.kneighbors(processed_query)
@@ -314,8 +360,18 @@ class NeighborsSimilarity(BaseSimilarity):
 
 
 class Word2VecSimilarityBase(BaseSimilarity):
-    def __init__(self, cut_off=0.2, cleanup_urls=True, nltk_tokenizer=False):
-        super().__init__(cleanup_urls=cleanup_urls, nltk_tokenizer=nltk_tokenizer)
+    def __init__(
+        self,
+        cut_off=0.2,
+        cleanup_urls=True,
+        nltk_tokenizer=False,
+        confidence_threshold=0.8,
+    ):
+        super().__init__(
+            cleanup_urls=cleanup_urls,
+            nltk_tokenizer=nltk_tokenizer,
+            confidence_threshold=confidence_threshold,
+        )
         self.corpus = []
         self.bug_ids = []
         self.cut_off = cut_off
@@ -333,12 +389,22 @@ class Word2VecSimilarityBase(BaseSimilarity):
 
 
 class Word2VecWmdSimilarity(Word2VecSimilarityBase):
-    def __init__(self, cut_off=0.2, cleanup_urls=True, nltk_tokenizer=False):
-        super().__init__(cleanup_urls=cleanup_urls, nltk_tokenizer=nltk_tokenizer)
+    def __init__(
+        self,
+        cut_off=0.2,
+        cleanup_urls=True,
+        nltk_tokenizer=False,
+        confidence_threshold=0.8,
+    ):
+        super().__init__(
+            cleanup_urls=cleanup_urls,
+            nltk_tokenizer=nltk_tokenizer,
+            confidence_threshold=confidence_threshold,
+        )
 
     # word2vec.wmdistance calculates only the euclidean distance. To get the cosine distance,
     # we're using the function with a few subtle changes. We compute the cosine distances
-    # in the get_similar_bugs method and use this inside the wmdistance method.
+    # in the search_similar_bugs method and use this inside the wmdistance method.
     def wmdistance(self, document1, document2, all_distances, distance_metric="cosine"):
         model = self.w2vmodel
         if len(document1) == 0 or len(document2) == 0:
@@ -397,7 +463,7 @@ class Word2VecWmdSimilarity(Word2VecSimilarityBase):
             dtype=np.double,
         )
 
-    def get_similar_bugs(self, query):
+    def search_similar_bugs(self, query):
 
         words = self.text_preprocess(self.get_text(query))
         words = [word for word in words if word in self.w2vmodel.wv.vocab]
@@ -466,12 +532,22 @@ class Word2VecWmdSimilarity(Word2VecSimilarityBase):
 
 
 class Word2VecWmdRelaxSimilarity(Word2VecSimilarityBase):
-    def __init__(self, cut_off=0.2, cleanup_urls=True, nltk_tokenizer=False):
-        super().__init__(cleanup_urls=cleanup_urls, nltk_tokenizer=nltk_tokenizer)
+    def __init__(
+        self,
+        cut_off=0.2,
+        cleanup_urls=True,
+        nltk_tokenizer=False,
+        confidence_threshold=0.8,
+    ):
+        super().__init__(
+            cleanup_urls=cleanup_urls,
+            nltk_tokenizer=nltk_tokenizer,
+            confidence_threshold=confidence_threshold,
+        )
         self.dictionary = Dictionary(self.corpus)
         self.tfidf = TfidfModel(dictionary=self.dictionary)
 
-    def get_similar_bugs(self, query):
+    def search_similar_bugs(self, query):
 
         query = self.text_preprocess(self.get_text(query))
         words = [
@@ -564,8 +640,18 @@ class Word2VecWmdRelaxSimilarity(Word2VecSimilarityBase):
 
 
 class Word2VecSoftCosSimilarity(Word2VecSimilarityBase):
-    def __init__(self, cut_off=0.2, cleanup_urls=True, nltk_tokenizer=False):
-        super().__init__(cleanup_urls=cleanup_urls, nltk_tokenizer=nltk_tokenizer)
+    def __init__(
+        self,
+        cut_off=0.2,
+        cleanup_urls=True,
+        nltk_tokenizer=False,
+        confidence_threshold=0.8,
+    ):
+        super().__init__(
+            cleanup_urls=cleanup_urls,
+            nltk_tokenizer=nltk_tokenizer,
+            confidence_threshold=confidence_threshold,
+        )
 
         terms_idx = WordEmbeddingSimilarityIndex(self.w2vmodel.wv)
         self.dictionary = Dictionary(self.corpus)
@@ -577,7 +663,7 @@ class Word2VecSoftCosSimilarity(Word2VecSimilarityBase):
             bow, similarity_matrix, num_best=10
         )
 
-    def get_similar_bugs(self, query):
+    def search_similar_bugs(self, query):
         similarities = self.softcosinesimilarity[
             self.dictionary.doc2bow(self.text_preprocess(self.get_text(query)))
         ]
@@ -592,8 +678,14 @@ class Word2VecSoftCosSimilarity(Word2VecSimilarityBase):
 
 
 class BM25Similarity(BaseSimilarity):
-    def __init__(self, cleanup_urls=True, nltk_tokenizer=False):
-        super().__init__(cleanup_urls=cleanup_urls, nltk_tokenizer=nltk_tokenizer)
+    def __init__(
+        self, cleanup_urls=True, nltk_tokenizer=False, confidence_threshold=0.8
+    ):
+        super().__init__(
+            cleanup_urls=cleanup_urls,
+            nltk_tokenizer=nltk_tokenizer,
+            confidence_threshold=confidence_threshold,
+        )
         self.corpus = []
         self.bug_ids = []
 
@@ -608,7 +700,7 @@ class BM25Similarity(BaseSimilarity):
 
         self.model = BM25(self.corpus)
 
-    def get_similar_bugs(self, query):
+    def search_similar_bugs(self, query):
         distances = self.model.get_scores(self.text_preprocess(self.get_text(query)))
         id_dist = zip(self.bug_ids, distances)
 
@@ -621,8 +713,14 @@ class BM25Similarity(BaseSimilarity):
 
 
 class LDASimilarity(BaseSimilarity):
-    def __init__(self, cleanup_urls=True, nltk_tokenizer=False):
-        super().__init__(cleanup_urls=cleanup_urls, nltk_tokenizer=nltk_tokenizer)
+    def __init__(
+        self, cleanup_urls=True, nltk_tokenizer=False, confidence_threshold=0.8
+    ):
+        super().__init__(
+            cleanup_urls=cleanup_urls,
+            nltk_tokenizer=nltk_tokenizer,
+            confidence_threshold=confidence_threshold,
+        )
         self.corpus = []
         self.bug_ids = []
         for bug in bugzilla.get_bugs():
@@ -638,7 +736,7 @@ class LDASimilarity(BaseSimilarity):
 
         self.model = LdaModel([self.dictionary.doc2bow(text) for text in self.corpus])
 
-    def get_similar_bugs(self, query):
+    def search_similar_bugs(self, query):
         query = self.text_preprocess(self.get_text(query))
 
         dense1 = sparse2full(
@@ -666,8 +764,14 @@ class LDASimilarity(BaseSimilarity):
 
 
 class ElasticSearchSimilarity(BaseSimilarity):
-    def __init__(self, cleanup_urls=True, nltk_tokenizer=False):
-        super().__init__(cleanup_urls=cleanup_urls, nltk_tokenizer=nltk_tokenizer)
+    def __init__(
+        self, cleanup_urls=True, nltk_tokenizer=False, confidence_threshold=0.8
+    ):
+        super().__init__(
+            cleanup_urls=cleanup_urls,
+            nltk_tokenizer=nltk_tokenizer,
+            confidence_threshold=confidence_threshold,
+        )
         self.elastic_search = Elasticsearch()
         assert (
             self.elastic_search.ping()
@@ -688,7 +792,7 @@ class ElasticSearchSimilarity(BaseSimilarity):
         self.elastic_search.indices.delete(index="bugbug", ignore=[400, 404])
         bulk(self.elastic_search, self.make_documents())
 
-    def get_similar_bugs(self, query):
+    def search_similar_bugs(self, query):
         find_similar = self.text_preprocess(
             self.get_text(query, all_comments=True), stemming=False, join=True
         )
