@@ -14,11 +14,13 @@ import tarfile
 from collections import deque
 from contextlib import contextmanager
 from functools import lru_cache
+from typing import Any, List
 
 import boto3
 import dateutil.parser
 import lmdb
 import numpy as np
+import psutil
 import requests
 import scipy
 import taskcluster
@@ -360,7 +362,7 @@ class ExpQueue:
 
 
 class LMDBDict:
-    def __init__(self, path, readonly=False):
+    def __init__(self, path: str, readonly: bool = False):
         self.readonly = readonly
         self.db = lmdb.open(
             path,
@@ -372,22 +374,22 @@ class LMDBDict:
         )
         self.txn = self.db.begin(buffers=True, write=not readonly)
 
-    def close(self):
+    def close(self) -> None:
         self.txn.commit()
         if not self.readonly:
             self.db.sync()
         self.db.close()
 
-    def __contains__(self, key):
+    def __contains__(self, key: bytes) -> bool:
         return self.txn.get(key) is not None
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: bytes) -> Any:
         val = self.txn.get(key)
         if val is None:
             raise KeyError
         return val
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: bytes, value: Any) -> None:
         self.txn.put(key, value, dupdata=False)
 
 
@@ -438,7 +440,7 @@ def get_session(name):
     return session
 
 
-def get_hgmo_stack(branch: str, revision: str) -> list:
+def get_hgmo_stack(branch: str, revision: str) -> List[bytes]:
     """Load descriptions of patches in the stack for a given revision"""
     url = f"https://hg.mozilla.org/{branch}/json-automationrelevance/{revision}"
     r = get_session("hgmo").get(url)
@@ -464,4 +466,10 @@ def get_hgmo_stack(branch: str, revision: str) -> list:
 
         return False
 
-    return [c for c in r.json()["changesets"] if not should_skip(c)]
+    return [
+        c["node"].encode("ascii") for c in r.json()["changesets"] if not should_skip(c)
+    ]
+
+
+def get_physical_cpu_count() -> int:
+    return psutil.cpu_count(logical=False)
